
// Copyright (c) 2009 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// analysis framework. 
// for actual examples of analysis engines, see analysis-engines.js

(function(JV3) {
    
    JV3.Analysis = {
	engines: {},
	engine_instances:{},
	interaction:{},
	initialize:function() {
	    var this_ = this;
	    this.log = JV3.log;
	    this.util = JV3.plumutil;
	    this.timer_interval = 100;
	    this.analysis_queue = [];
	    this.plumutil = JV3.plumutil;
	    this.engine_instances = this.plumutil.objMap(
		function(eng) { return JV3.newify(eng); },
		this.engines);
	    this._start();
	},
	_start: function() {
	    var this_ = this;
	    var timercb = function() {
		try {
		    var noteid = this_.analysis_queue.shift();
		    if (!noteid) { return; }
		    this_.log("analysis -- Analyzing note : " + noteid);
		    var note = JV3.getNoteById(noteid);
		    this_._analyzeNoteHelper(note); // no need to save cuz we're going to save again
		    JV3.remoteUpdate(note);				
		} catch(e) {
		    this_.log(e + "  " + e.lineNumber + " " + e.fileName);
		}		
	    };
	    this._timer = JV3.setInterval( timercb, this.timer_interval );				   
	},
	appendResult : function(results,action) {
	    var start = action.start;
	    var end = action.end;
	    // keeps all intepretations
	    if (results[start] === undefined) {
		results[start] = [];
	    }
	    results[start].push(action);
	},
	_hitoverlapsWith:function(hit1) {
	    return function(hit2) {
		return hit1.end > hit2.start && hit2.end > hit1.start;
	    };
	},    
	_save_Analysis: function(note,results) {
	    var jsonr = this.plumutil.toJSON(results);
	    note.analysis = jsonr;
	    return jsonr;
	},
	_load_Analysis: function(note) {
	    try {
		return note.analysis ? this.plumutil.fromJSON(note.analysis) : undefined;
	    } catch(e) {
		this.log(e + " " + e.lineNumber + " " + e.fileName);
	    }
	},    
	merge_results_choose_maximum_spanning_parses : function(results,action) {
	    var objMap = this.plumutil.objMap, objFilter = this.plumutil.objFilter, keys = this.plumutil.keys, objVals = this.plumutil.objVals;
	    var start = action.start;
	    var end = action.end;
	    //this.log("action : " + start + "-" + end + " ");
	    var textspan = function(ent) { return ent.end - ent.start; };
	    var overlaps = objFilter(this._hitoverlapsWith(action),results);
	    // only replace if we dominate the set
	    if (keys(overlaps).length === 0 || Math.max(objVals(objMap(textspan,overlaps))) < textspan(action)) {
		objMap(function(x) { results[x.start] = undefined; }, overlaps);
		results[action.start] = action;
	    }
	},
	filter_over_substrings: function(test_fn, string_to_parse, success_fn, options) {
	    // takes a string and applies test_fn to all consecutive substrings
	    // for example, "foo bar baz" ->
	    // fn("foo"), fn("foo bar"), fn("foo bar baz"), fn("bar"), f("bar baz"), f("baz")
	    // and filters successes to success_fn(text,parse)
	    var this_ = this;
	    var results = [];		
	    var curpos = 0;
	    var maxlength = options && options.maxlength ? options.maxlength : 1000;
	    var remain = string_to_parse.substring(0,maxlength);
	    while (remain.length > 0) {
		// end paragraph marks the end of our destination
		var endparapos = remain.indexOf('\n');
		if (endparapos == -1) { endparapos = remain.length; }
		var parabuff = remain.substring(0,endparapos);
		
		// inchworm algorithm :
		// |foo|
		// |foo bar|
		// |foo bar baz|
		// bar
		// bar baz
		// baz
		
		var eatstart = 0, eatend = 0;	    
		while (eatstart < parabuff.length) {
		    eatend = eatstart;
		    // advance eatend one at a time until the end			
		    while (eatend < parabuff.length) {
			var nextend = parabuff.indexOf(' ',eatend+1);
			if (nextend == -1) { nextend = parabuff.length; }
			eatend = nextend;			    
			var buff = parabuff.substring(eatstart,eatend);
			// -- JV3.log("BUFF is  \"" + buff+"\"");
			var result = test_fn(curpos+eatstart,curpos+eatend,buff);
			if (result) {  results.push(success_fn(buff,result));  }
			// -- JV3.log("inner loop curpos " + curpos + " eatstart : " + eatstart + " eatend : " + eatend);
		    }			
		    var nextstart = parabuff.indexOf(' ',eatstart+1);
		    if (nextstart == -1) { nextstart = parabuff.length; }
		    eatstart = nextstart + 1;
		    // -- JV3.log("curpos " + curpos + " eatstart : " + eatstart + " eatend : " + eatend);
		}
		// -- JV3.log("parabuff " + parabuff);
		remain = remain.substring(endparapos + 1);
		curpos = curpos + endparapos + 1;
		// -- JV3.log(" remain : " + remain + " endparapos " + endparapos);
	    }
	    // -- JV3.log("EXIT!");
	    return results;
	},    
	addEngine: function(name,defnition) {
	    this.engines[name] = defnition;
	    this.engine_instances[name] = JV3.newify(defnition,this);
	},
	analyze: function(text) {
	    this.log("JV3.Analysis(): analyzing text: " + text.substring(0,10));
	    var this_ = this;
	    var results = {};	
	    this.plumutil.objMap( function(engine) {	    
		try {
		    this_.log("JV3.Analysis() running engine >" + engine.name + " on text[" + text.length + "]");
		    var eng_results = engine.analyze(text);
     		    try { this_.log("Analysis() : results " + (typeof eng_results) + " " + eng_results.length); } catch(e2) { this_.log(e2); }
		    for (var res_idx in eng_results) {
			var interps = !JV3.plumutil.isArrayLike(eng_results[res_idx]) ? [eng_results[res_idx]] : eng_results[res_idx];
			interps.map( function(r) {
			    //this_.log("hai."); try {      this_.log("JV3.Analysis() !!!! result : " r !== undefined);  } catch(e) { this_.log(e + " " + e.lineNumber); }
			    this_.merge_results_choose_maximum_spanning_parses(results,r); 
			} );
		    }
		} catch(e) { this_.log("ANALYSIS ERROR" + e + " " + e.lineNumber + " " + e.fileName);    }
	    },  this.engine_instances);
	    return results;	
	},
	_analyzeNoteHelper:function(note) {
	    var text = note.text;	
	    this._save_Analysis( note, { hits: this.analyze(text), text : text } );
	    return note.analysis;
	},
	_enqueue_for_Analysis:function(note){
	    if (this.analysis_queue.indexOf(note.id) < 0) {
		this.analysis_queue.push(note.id);
	    }
	},
	getAnalysis:function(note, supress_auto_queue) {
	    // if analysis is readily available, we'll give it to you.
	    // otherwise, we'll return an undefined and queue it for analysis
	    var note_analysis = this._load_Analysis(note);
	    //this.log("NOTE ANLAYSIS TEXT " + note_analysis + "TEXT: \"" + (note_analysis ? note_analysis.text : "") + "\" HITS:" + (note_analysis ? note_analysis.hits : ""));
	    if (note_analysis !== undefined && note_analysis.text == note.text) {
		//this.log("note_analysis matches: " + note_analysis.text);
		return note_analysis;
	    }
	    if (!supress_auto_queue) { this._enqueue_for_Analysis(note); }
	},
	initDisplay:function(display_env) {
	    /* this function is called by the view to declare ourselves a new display.create() function
           and replace the existing one */
	    var this_ = this;
	    var old_display = display_env.display.create;
	    var JV3 = display_env.getJV3();
	    var objMap = JV3.plumutil.objMap, objKeys = JV3.plumutil.objKeys;

	    var illuminate_type = function(document, note, params) {
		//this_.log("in illuminate_type!");
		try { 
		    var note_analysis = this_.getAnalysis(note);
		    if (note_analysis === undefined) { return; }
		    var textSpan = params.textspan;
		    var text = note.text;
		    var string_to_semantics = {};		
		    objMap(function(entity) {
			if (entity.start !== undefined && entity.end !== undefined) {
			    string_to_semantics[ note.text.substring( entity.start, entity.end ) ] = { type: entity.type, value: entity.value } ;
			}
		    }, note_analysis.hits);
		    var apply_semantics = function(text) {
			var event_handlers = objKeys(string_to_semantics).map(
			    function(string) {
				var semantics = string_to_semantics[string];
				var rendering = this_.interaction.lens.getTextSpanLens(semantics.type, string, semantics);
				if (rendering) {
				    var html = rendering[0];
				    var evts = rendering[1];
				    try {
					text = plumutil.replaceAll(text,string,html);//text.replace(new RegExp(string,"g"),html);
					return evts;
				    } catch(e) {
					JV3.log(e);
				    }
				} else {
				    this_.log("Problem with illuminating type: " + semantics.type);
				}
				return function() {};
			    });
			return [text,event_handlers];
		    };	    
		    var html_handlers = apply_semantics(JV3.util.defang(text));
		    textSpan.innerHTML = html_handlers[0] ? html_handlers[0] : "";
		    if (html_handlers[1]) { html_handlers[1].map(function(fn_e) { fn_e(document,textSpan); }); }
		} catch(e) {
		    this_.log("Error : " + e + " " + e.lineNumber + " " + e.fileName);
		}
	    };
	    // add to displayNote hook.
	    display_env.addAugmentation("displayNote", illuminate_type);
	}
    };

    try {
	JV3.xuload("chrome://listit/content/analysis/engines.js")(JV3,JV3.Analysis);
	JV3.xuload("chrome://listit/content/analysis/interaction.js")(JV3,JV3.Analysis,JV3.Analysis.interaction);
    } catch(e) {
	JV3.log(e + " " + e.lineNumber + " " + e.fileName);
    }    
});
